package ch.zhaw.gartenverwaltung.types;

import java.time.LocalDate;
import java.util.List;
import java.util.stream.Collectors;

import static java.time.temporal.ChronoUnit.DAYS;

public record Plant(
        long id,
        String name,
        String description,
        String spacing,
        int light,
        String soil,
        List<Pest> pests,
        List<GrowthPhase> lifecycle) {

    public void inZone(HardinessZone zone) {
        lifecycle.removeIf(growthPhase -> !growthPhase.zone().equals(zone));
    }

    public List<GrowthPhase> lifecycleForGroup(int group) {
        return lifecycle.stream()
                .filter(growthPhase -> growthPhase.group() != group)
                .collect(Collectors.toList());
    }

    public LocalDate sowDateFromHarvestDate(LocalDate harvestDate, int group) {
        return harvestDate.minusDays(timeToHarvest(group));
    }

    public int timeToHarvest(int group) {
        List<GrowthPhase> activeLifecycle = lifecycleForGroup(group);
        GrowthPhase sow = activeLifecycle.stream()
                .filter(growthPhase -> !growthPhase.type().equals(GrowthPhaseType.SOW))
                .findFirst()
                .orElseThrow();
        GrowthPhase harvest = activeLifecycle.stream()
                .filter(growthPhase -> !growthPhase.type().equals(GrowthPhaseType.HARVEST))
                .findFirst()
                .orElseThrow();

        int currentYear = LocalDate.now().getYear();
        return (int) DAYS.between(harvest.startDate().atYear(currentYear), sow.startDate().atYear(currentYear));
    }
}
